@@ -1,9 +1,9 @@
 import os
+import re
 import sys
 
 from pipenv import environments
 from pipenv.__version__ import __version__
-from pipenv._compat import fix_utf8
 from pipenv.cli.options import (
     CONTEXT_SETTINGS,
     PipenvGroup,
@@ -22,6 +22,7 @@ from pipenv.cli.options import (
     uninstall_options,
     verbose_option,
 )
+from pipenv.utils.dependencies import get_lockfile_section_using_pipfile_category
 from pipenv.utils.environment import load_dot_env
 from pipenv.utils.processes import subprocess_run
 from pipenv.vendor.click import (
@@ -478,13 +479,23 @@ def run(state, command, args):
     help="Path to where output file will be placed, if the path is a directory, "
     "Safety will use safety-report.json as filename. Default: empty",
 )
+@option(
+    "--use-installed",
+    is_flag=True,
+    help="Whether to use the lockfile as input to check (instead of result from pip list).",
+)
+@option(
+    "--categories",
+    is_flag=False,
+    default="",
+    help="Use the specified categories from the lockfile as input to check.",
+)
 @common_options
 @system_option
 @pass_state
 def check(
     state,
     db=None,
-    style=False,
     ignore=None,
     output="screen",
     key=None,
@@ -494,6 +505,8 @@ def check(
     save_json="",
     audit_and_monitor=True,
     project=None,
+    use_installed=False,
+    categories="",
     **kwargs,
 ):
     """Checks for PyUp Safety security vulnerabilities and against PEP 508 markers provided in Pipfile."""
@@ -514,6 +527,8 @@ def check(
         audit_and_monitor=audit_and_monitor,
         safety_project=project,
         pypi_mirror=state.pypi_mirror,
+        use_installed=use_installed,
+        categories=categories,
     )
 
 
@@ -725,7 +740,7 @@ def scripts(state):
         "{0:<{width}}  {1}".format(name, script, width=first_column_width)
         for name, script in scripts.items()
     )
-    echo("\n".join(fix_utf8(line) for line in lines))
+    echo("\n".join(line for line in lines))
 
 
 @cli.command(
@@ -770,7 +785,6 @@ def verify(state):
 def requirements(
     state, dev=False, dev_only=False, hash=False, exclude_markers=False, categories=""
 ):
-
     from pipenv.utils.dependencies import convert_deps_to_pip
 
     lockfile = state.project.load_lockfile(expand_env_vars=False)
@@ -780,11 +794,11 @@ def requirements(
         echo(" ".join([prefix, package_index["url"]]))
 
     deps = {}
-    categories_list = categories.split(",") if categories else []
+    categories_list = re.split(r", *| ", categories) if categories else []
 
     if categories_list:
         for category in categories_list:
-            category = category.strip()
+            category = get_lockfile_section_using_pipfile_category(category.strip())
             deps.update(lockfile.get(category, {}))
     else:
         if dev or dev_only:
