@@ -12,6 +12,7 @@ import posixpath
 import shutil
 import stat
 import sys
+import sysconfig
 import urllib.parse
 from io import StringIO
 from itertools import filterfalse, tee, zip_longest
@@ -34,11 +35,11 @@ from typing import (
     cast,
 )
 
-from pipenv.patched.pip._vendor.pep517 import Pep517HookCaller
+from pipenv.patched.pip._vendor.pyproject_hooks import BuildBackendHookCaller
 from pipenv.patched.pip._vendor.tenacity import retry, stop_after_delay, wait_fixed
 
 from pipenv.patched.pip import __version__
-from pipenv.patched.pip._internal.exceptions import CommandError
+from pipenv.patched.pip._internal.exceptions import CommandError, ExternallyManagedEnvironment
 from pipenv.patched.pip._internal.locations import get_major_minor_version
 from pipenv.patched.pip._internal.utils.compat import WINDOWS
 from pipenv.patched.pip._internal.utils.virtualenv import running_under_virtualenv
@@ -57,10 +58,10 @@ __all__ = [
     "captured_stdout",
     "ensure_dir",
     "remove_auth_from_url",
-    "ConfiguredPep517HookCaller",
+    "check_externally_managed",
+    "ConfiguredBuildBackendHookCaller",
 ]
 
-
 logger = logging.getLogger(__name__)
 
 T = TypeVar("T")
@@ -581,6 +582,21 @@ def protect_pip_from_modification_on_windows(modifying_pip: bool) -> None:
         )
 
 
+def check_externally_managed() -> None:
+    """Check whether the current environment is externally managed.
+
+    If the ``EXTERNALLY-MANAGED`` config file is found, the current environment
+    is considered externally managed, and an ExternallyManagedEnvironment is
+    raised.
+    """
+    if running_under_virtualenv():
+        return
+    marker = os.path.join(sysconfig.get_path("stdlib"), "EXTERNALLY-MANAGED")
+    if not os.path.isfile(marker):
+        return
+    raise ExternallyManagedEnvironment.from_config(marker)
+
+
 def is_console_interactive() -> bool:
     """Is this console interactive?"""
     return sys.stdin is not None and sys.stdin.isatty()
@@ -603,7 +619,7 @@ def is_wheel_installed() -> bool:
     Return whether the wheel package is installed.
     """
     try:
-        import pipenv.vendor.wheel as wheel  # noqa: F401
+        import wheel  # noqa: F401
     except ImportError:
         return False
 
@@ -635,7 +651,7 @@ def partition(
     return filterfalse(pred, t1), filter(pred, t2)
 
 
-class ConfiguredPep517HookCaller(Pep517HookCaller):
+class ConfiguredBuildBackendHookCaller(BuildBackendHookCaller):
     def __init__(
         self,
         config_holder: Any,
